# import json

from web3 import Web3

abi = [
    {
        "anonymous": False,
        "inputs": [
            {
                "indexed": True,
                "internalType": "address",
                "name": "previousOwner",
                "type": "address"
            },
            {
                "indexed": True,
                "internalType": "address",
                "name": "newOwner",
                "type": "address"
            }
        ],
        "name": "OwnershipTransferred",
        "type": "event"
    },
    {
        "inputs": [
            {
                "internalType": "string",
                "name": "_hash",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "_certNum",
                "type": "string"
            },
            {
                "internalType": "uint256",
                "name": "_expireDate",
                "type": "uint256"
            },
            {
                "internalType": "string",
                "name": "_version",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "_desc",
                "type": "string"
            }
        ],
        "name": "addCertification",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "string",
                "name": "",
                "type": "string"
            }
        ],
        "name": "certifications",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "internalType": "string",
                "name": "certNum",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "hash",
                "type": "string"
            },
            {
                "internalType": "address",
                "name": "issuer",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "expireDate",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "createdAt",
                "type": "uint256"
            },
            {
                "internalType": "bool",
                "name": "isRevoked",
                "type": "bool"
            },
            {
                "internalType": "string",
                "name": "version",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "description",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "revokerName",
                "type": "string"
            },
            {
                "internalType": "uint256",
                "name": "revokedAt",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "addr",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "credit",
                "type": "uint256"
            }
        ],
        "name": "chargeCredit",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "creditAddress",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "name": "credits",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "string",
                "name": "hash",
                "type": "string"
            }
        ],
        "name": "getCertification",
        "outputs": [
            {
                "components": [
                    {
                        "internalType": "uint256",
                        "name": "id",
                        "type": "uint256"
                    },
                    {
                        "internalType": "string",
                        "name": "certNum",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "hash",
                        "type": "string"
                    },
                    {
                        "internalType": "address",
                        "name": "issuer",
                        "type": "address"
                    },
                    {
                        "internalType": "uint256",
                        "name": "expireDate",
                        "type": "uint256"
                    },
                    {
                        "internalType": "uint256",
                        "name": "createdAt",
                        "type": "uint256"
                    },
                    {
                        "internalType": "bool",
                        "name": "isRevoked",
                        "type": "bool"
                    },
                    {
                        "internalType": "string",
                        "name": "version",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "description",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "revokerName",
                        "type": "string"
                    },
                    {
                        "internalType": "uint256",
                        "name": "revokedAt",
                        "type": "uint256"
                    }
                ],
                "internalType": "struct CertificationRegistration.Certification",
                "name": "",
                "type": "tuple"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "string",
                "name": "certNum",
                "type": "string"
            }
        ],
        "name": "getCertificationByCertNum",
        "outputs": [
            {
                "components": [
                    {
                        "internalType": "uint256",
                        "name": "id",
                        "type": "uint256"
                    },
                    {
                        "internalType": "string",
                        "name": "certNum",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "hash",
                        "type": "string"
                    },
                    {
                        "internalType": "address",
                        "name": "issuer",
                        "type": "address"
                    },
                    {
                        "internalType": "uint256",
                        "name": "expireDate",
                        "type": "uint256"
                    },
                    {
                        "internalType": "uint256",
                        "name": "createdAt",
                        "type": "uint256"
                    },
                    {
                        "internalType": "bool",
                        "name": "isRevoked",
                        "type": "bool"
                    },
                    {
                        "internalType": "string",
                        "name": "version",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "description",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "revokerName",
                        "type": "string"
                    },
                    {
                        "internalType": "uint256",
                        "name": "revokedAt",
                        "type": "uint256"
                    }
                ],
                "internalType": "struct CertificationRegistration.Certification",
                "name": "",
                "type": "tuple"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "ID",
                "type": "uint256"
            }
        ],
        "name": "getCertificationByID",
        "outputs": [
            {
                "components": [
                    {
                        "internalType": "uint256",
                        "name": "id",
                        "type": "uint256"
                    },
                    {
                        "internalType": "string",
                        "name": "certNum",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "hash",
                        "type": "string"
                    },
                    {
                        "internalType": "address",
                        "name": "issuer",
                        "type": "address"
                    },
                    {
                        "internalType": "uint256",
                        "name": "expireDate",
                        "type": "uint256"
                    },
                    {
                        "internalType": "uint256",
                        "name": "createdAt",
                        "type": "uint256"
                    },
                    {
                        "internalType": "bool",
                        "name": "isRevoked",
                        "type": "bool"
                    },
                    {
                        "internalType": "string",
                        "name": "version",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "description",
                        "type": "string"
                    },
                    {
                        "internalType": "string",
                        "name": "revokerName",
                        "type": "string"
                    },
                    {
                        "internalType": "uint256",
                        "name": "revokedAt",
                        "type": "uint256"
                    }
                ],
                "internalType": "struct CertificationRegistration.Certification",
                "name": "",
                "type": "tuple"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "addr",
                "type": "address"
            }
        ],
        "name": "getCredit",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "id",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "initialize",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "string",
                "name": "",
                "type": "string"
            }
        ],
        "name": "mapByCertNum",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "internalType": "string",
                "name": "certNum",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "hash",
                "type": "string"
            },
            {
                "internalType": "address",
                "name": "issuer",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "expireDate",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "createdAt",
                "type": "uint256"
            },
            {
                "internalType": "bool",
                "name": "isRevoked",
                "type": "bool"
            },
            {
                "internalType": "string",
                "name": "version",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "description",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "revokerName",
                "type": "string"
            },
            {
                "internalType": "uint256",
                "name": "revokedAt",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "",
                "type": "uint256"
            }
        ],
        "name": "mapById",
        "outputs": [
            {
                "internalType": "uint256",
                "name": "id",
                "type": "uint256"
            },
            {
                "internalType": "string",
                "name": "certNum",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "hash",
                "type": "string"
            },
            {
                "internalType": "address",
                "name": "issuer",
                "type": "address"
            },
            {
                "internalType": "uint256",
                "name": "expireDate",
                "type": "uint256"
            },
            {
                "internalType": "uint256",
                "name": "createdAt",
                "type": "uint256"
            },
            {
                "internalType": "bool",
                "name": "isRevoked",
                "type": "bool"
            },
            {
                "internalType": "string",
                "name": "version",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "description",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "revokerName",
                "type": "string"
            },
            {
                "internalType": "uint256",
                "name": "revokedAt",
                "type": "uint256"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "owner",
        "outputs": [
            {
                "internalType": "address",
                "name": "",
                "type": "address"
            }
        ],
        "stateMutability": "view",
        "type": "function"
    },
    {
        "inputs": [],
        "name": "renounceOwnership",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "string",
                "name": "hash",
                "type": "string"
            },
            {
                "internalType": "string",
                "name": "revokerName",
                "type": "string"
            }
        ],
        "name": "revoke",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "uint256",
                "name": "ID",
                "type": "uint256"
            },
            {
                "internalType": "string",
                "name": "revokerName",
                "type": "string"
            }
        ],
        "name": "revokeById",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    },
    {
        "inputs": [
            {
                "internalType": "address",
                "name": "newOwner",
                "type": "address"
            }
        ],
        "name": "transferOwnership",
        "outputs": [],
        "stateMutability": "nonpayable",
        "type": "function"
    }
]

DEFAULT_GAS_LIMIT = 2000000


def add_certification(hash_str: str, cert_num: str, expire_date: int, version: str, desc: str, node_url: str,
                      address: str, contract_address: str, pk: str):
    client = Web3(Web3.HTTPProvider(node_url))
    contract_instance = client.eth.contract(address=contract_address, abi=abi)
    nonce = client.eth.getTransactionCount(client.toChecksumAddress(address))

    try:
        func = contract_instance.functions.addCertification(hash_str, cert_num, expire_date, version, desc)
        tx = func.buildTransaction(
            {'from': address, 'gasPrice': client.toWei('1000', 'gwei'), 'nonce': nonce, 'gas': DEFAULT_GAS_LIMIT})
        signed = client.eth.account.signTransaction(tx, pk)
        tx_hash = client.eth.sendRawTransaction(signed.rawTransaction)
        tx_res = client.eth.wait_for_transaction_receipt(tx_hash)
        if tx_res.status == 1:
            return client.toHex(tx_hash), None
        return '', 'Failed on blockchain'
    except Exception as e:
        print(e)
        return '', e


def revoke_certification(hash_str: str, revoker_name: str, node_url: str, contract_address: str, address: str, pk: str):
    client = Web3(Web3.HTTPProvider(node_url))
    contract_instance = client.eth.contract(address=contract_address, abi=abi)
    nonce = client.eth.getTransactionCount(client.toChecksumAddress(address))

    try:
        func = contract_instance.functions.revoke(hash_str, revoker_name)
        tx = func.buildTransaction(
            {'from': address, 'gasPrice': client.toWei('1000', 'gwei'), 'nonce': nonce, 'gas': DEFAULT_GAS_LIMIT})
        signed = client.eth.account.signTransaction(tx, pk)
        tx_hash = client.eth.sendRawTransaction(signed.rawTransaction)
        tx_res = client.eth.wait_for_transaction_receipt(tx_hash)
        if tx_res.status == 1:
            return client.toHex(tx_hash), None
        return '', 'Failed on blockchain'
    except Exception as e:
        print(e)
        return '', e


def revoke_certification_by_id(cert_id: int, revoker_name: str, node_url: str, contract_address: str, address: str,
                               pk: str):
    client = Web3(Web3.HTTPProvider(node_url))
    # f = open('abi.json')
    # abi = json.load(f)
    contract_instance = client.eth.contract(address=contract_address, abi=abi)
    nonce = client.eth.getTransactionCount(client.toChecksumAddress(address))

    try:
        func = contract_instance.functions.revokeById(cert_id, revoker_name)
        tx = func.buildTransaction(
            {'from': address, 'gasPrice': client.toWei('1000', 'gwei'), 'nonce': nonce, 'gas': DEFAULT_GAS_LIMIT})
        signed = client.eth.account.signTransaction(tx, pk)
        tx_hash = client.eth.sendRawTransaction(signed.rawTransaction)
        tx_res = client.eth.wait_for_transaction_receipt(tx_hash)
        if tx_res.status == 1:
            return client.toHex(tx_hash), None
        return '', 'Failed on blockchain'
    except Exception as e:
        print(e)
        return '', e


def charge_credit(address: str, credit: int, owner_address, owner_pk, node_url, contract_address):
    client = Web3(Web3.HTTPProvider(node_url))
    # f = open('abi.json')
    # abi = json.load(f)
    contract_instance = client.eth.contract(address=contract_address, abi=abi)
    nonce = client.eth.getTransactionCount(client.toChecksumAddress(owner_address))

    try:
        func = contract_instance.functions.chargeCredit(client.toChecksumAddress(address), credit)
        tx = func.buildTransaction(
            {'from': address, 'gasPrice': client.toWei('1000', 'gwei'), 'nonce': nonce, 'gas': DEFAULT_GAS_LIMIT})
        signed = client.eth.account.signTransaction(tx, owner_pk)
        tx_hash = client.eth.sendRawTransaction(signed.rawTransaction)
        tx_res = client.eth.wait_for_transaction_receipt(tx_hash)
        if tx_res.status == 1:
            return client.toHex(tx_hash), None
        return '', 'Failed on blockchain'
    except Exception as e:
        print(e)
        return '', e


def get_credit(address: str, contract_address, node_url):
    client = Web3(Web3.HTTPProvider(node_url))
    contract_instance = client.eth.contract(address=contract_address, abi=abi)

    return contract_instance.functions.getCredit(
        client.toChecksumAddress(address)).call()


def get_certificate(hash_str: str, contract_address, node_url):
    client = Web3(Web3.HTTPProvider(node_url))
    contract_instance = client.eth.contract(address=contract_address, abi=abi)
    return contract_instance.functions.getCertification(hash_str).call()



from merkletools import MerkleTools

CHAINPOINT_CONTEXT = 'https://w3id.org/chainpoint/v2'
CHAINPOINT_HASH_TYPES = {'sha224': 'ChainpointSHA224v2',
                         'sha256': 'ChainpointSHA256v2',
                         'sha384': 'ChainpointSHA384v2',
                         'sha512': 'ChainpointSHA512v2',
                         'sha3_224': 'ChainpointSHA3-224v2',
                         'sha3_256': 'ChainpointSHA3-256v2',
                         'sha3_384': 'ChainpointSHA3-384v2',
                         'sha3_512': 'ChainpointSHA3-512v2'}
CHAINPOINT_ANCHOR_TYPES = {
    'corex': 'CorexDataMain',
    'corex_testnet': 'CorexDataTest'}  # , 'eth': 'ETHData'


class ChainPointV2(object):
    """
    Implements chainpoint v2 proof of existence approach
    """

    def __init__(self, hash_type="sha256"):
        self.hash_type = hash_type.lower()
        self.mk = MerkleTools(hash_type)

    '''Wraps merkletools methods'''

    def reset_tree(self):
        self.mk.reset_tree()

    def add_leaf(self, values, do_hash=False):
        self.mk.add_leaf(values, do_hash)

    def get_leaf(self, index):
        return self.mk.get_leaf(index)

    def get_leaf_count(self):
        return self.mk.get_leaf_count()

    def get_tree_ready_state(self):
        return self.mk.get_tree_ready_state()

    def make_tree(self):
        self.mk.make_tree()

    def get_merkle_root(self):
        return self.mk.get_merkle_root()

    def get_proof(self, index):
        return self.mk.get_proof(index)

    def validate_proof(self, proof, target_hash, merkle_root):
        return self.mk.validate_proof(proof, target_hash, merkle_root)

    def get_chainpoint_hash_type(self):
        return CHAINPOINT_HASH_TYPES[self.hash_type]

    def get_receipt(self, index, source_id, test_net=False):
        """
        Returns the chainpoint v2 blockchain receipt for specific leaf
        Currently only works for corexchain
        """
        # chain_type = utils.get_chain_type(chain, testnet)
        chain_type = 'corex_testnet' if test_net else 'corex'
        if chain_type is not None and self.get_tree_ready_state():
            return {
                "@context": CHAINPOINT_CONTEXT,
                "type": self.get_chainpoint_hash_type(),
                "targetHash": self.get_leaf(index),
                "merkleRoot": self.get_merkle_root(),
                "proof": self.get_proof(index),
                "anchors": [
                    {
                        "type": CHAINPOINT_ANCHOR_TYPES[chain_type],
                        "sourceId": source_id
                    }
                ]
            }
        else:
            return None



#
# pdf файлтай ажиллах хэсэг
#
import hashlib
from pdfrw import PdfWriter, PdfReader, PdfDict


def hash_certificates(cert_files):
    """
    Hashes (sha256) all files passed as an array and returns them as an array.
    """
    hashes = []
    for f in cert_files:
        with open(f, 'rb') as cert:
            hash_ = hashlib.sha256(cert.read()).hexdigest()
            hashes.append(hash_)
            print(hash_)

    return hashes


def add_metadata(src: str, dest: str, **kwargs):
    pdf = PdfReader(src)
    metadata_dict = PdfDict(**kwargs)
    pdf.Info.update(metadata_dict)
    PdfWriter().write(dest, pdf)



import json
import os

from web3.auto import w3
import certify_issuer.utils as Utils
from certify_issuer.certify_sc_utils import add_certification, get_credit, revoke_certification, get_certificate
from certify_issuer.pdf import add_metadata

DEFAULT_CERTIFY_ADDRESS = ""
DEFAULT_NODE_URL = ""
VERSION = "v1.0"


def issue(src_path: str,
          dest_path: str,
          cert_num: str,
          address: str,
          issuer_name: str,
          expire_date: int = 0,
          description: str = "",
          private_key: str = "",
          key_store="",
          passphrase: str = "",
          certify_address=DEFAULT_CERTIFY_ADDRESS,
          node_url=DEFAULT_NODE_URL,
          is_testnet=False):
    # validation
    if not os.path.exists(src_path) or not os.path.isfile(src_path):
        raise ValueError('Source path should be valid')

    if os.path.isdir(dest_path):
        raise ValueError('Destination path already exists')

    if not w3.isAddress(address):
        raise ValueError("Address is invalid")

    if not w3.isAddress(certify_address):
        raise ValueError("Certify address is invalid")

    pk = private_key
    if private_key == "":
        if os.path.isdir(key_store):
            path = os.path.join(key_store, w3.toChecksumAddress(address) + '.json')
            pk = Utils.decrypt_account(passphrase, path)
        elif os.path.isfile(key_store):
            pk = Utils.decrypt_account(passphrase, key_store)
        else:
            raise ValueError("Private key or key store file is required")

    # check credit
    if check_credit(address, certify_address, node_url) == 0:
        raise ValueError("Not enough credit")
    # insert meta data
    Utils.insert_metadata_to_certificate(src_path, dest_path, cert_num, issuer_address=address, issuer_name=issuer_name)
    # calc hash
    hash_str = Utils.calc_hash(dest_path)

    if check_revoked_hash(hash_str):
        raise ValueError("Certificate revoked")

    if check_hash_exists(hash_str):
        raise ValueError("Certificate already registered")

    cp = Utils.prepare_chainpoint_tree([hash_str])

    tx, error = add_certification(cp.get_merkle_root(), cert_num, expire_date, version=VERSION, desc=description,
                                  node_url=node_url,
                                  address=address, contract_address=certify_address, pk=pk)
    if error is not None:
        raise RuntimeError(error)

    # insert proof
    proof = json.dumps(cp.get_receipt(0, tx, is_testnet))
    add_metadata(dest_path, dest_path, chainpoint_proof=proof)

    return tx, None


def revoke(src_path: str,
           address: str,
           revoker_name: str,
           private_key: str = "",
           key_store="",
           passphrase: str = "",
           certify_address=DEFAULT_CERTIFY_ADDRESS,
           node_url=DEFAULT_NODE_URL):
    # validation
    if not os.path.exists(src_path) or not os.path.isfile(src_path):
        raise ValueError('Source path should be valid')

    pk = private_key
    if private_key == "":
        if os.path.isdir(key_store):
            path = os.path.join(key_store, w3.toChecksumAddress(address) + '.json')
            pk = Utils.decrypt_account(passphrase, path)
        elif os.path.isfile(key_store):
            pk = Utils.decrypt_account(passphrase, key_store)
        else:
            raise ValueError("Private key or key store file is required")

    # check credit
    if check_credit(address, certify_address, node_url) == 0:
        raise ValueError("Not enough credit")
    # calc hash
    temp_path = Utils.create_temporary_copy(src_path)
    add_metadata(temp_path, temp_path, chainpoint_proof='')
    hash_str = Utils.calc_hash(temp_path)
    cp = Utils.prepare_chainpoint_tree([hash_str])

    tx, error = revoke_certification(cp.get_merkle_root(), revoker_name, node_url=node_url,
                                     address=address, contract_address=certify_address, pk=pk)

    if error is not None:
        raise RuntimeError(error)

    return tx, None


def check_credit(address: str,
                 certify_address=DEFAULT_CERTIFY_ADDRESS,
                 node_url=DEFAULT_NODE_URL):
    return get_credit(address, certify_address, node_url)


def check_revoked_hash(hash_str,
                       certify_address=DEFAULT_CERTIFY_ADDRESS,
                       node_url=DEFAULT_NODE_URL):
    t = get_certificate(hash_str, certify_address, node_url)
    return t[6]


def check_hash_exists(hash_str,
                      certify_address=DEFAULT_CERTIFY_ADDRESS,
                      node_url=DEFAULT_NODE_URL):
    t = get_certificate(hash_str, certify_address, node_url)
    if t[0] > 0:
        return True
    return False


def generate_account(key_store_dir_path: str, verbose=False):
    acc = w3.eth.account.create()
    passphrase = Utils.random_passphrase(20)
    keystore = acc.encrypt(passphrase)
    with open(os.path.join(key_store_dir_path, acc.address + '.json'), 'w') as outfile:
        json.dump(keystore, outfile)

    if verbose:
        print("New address generated: ", acc.address)

    return acc.address, passphrase



import hashlib
import json
import os
import random
import shutil
import string
import tempfile

import certify_issuer.pdf as pdf_utils
from certify_issuer.chainpoint import ChainPointV2


def random_passphrase(length=8):
    lower = string.ascii_lowercase
    upper = string.ascii_uppercase
    num = string.digits
    symbols = string.punctuation
    all = lower + upper + num + symbols
    temp = random.sample(all, length)
    return "".join(temp)


def decrypt_account(passphrase: str, path: str):
    from web3.auto import w3
    with open(path) as keyfile:
        encrypted_key = keyfile.read()
        private_key = w3.eth.account.decrypt(encrypted_key, passphrase)
        return w3.toHex(private_key)


def insert_metadata_to_certificate(src_path, dest_path, cert_num, issuer_name, issuer_address):
    issuer = {
        "name": issuer_name,
        "identity": {
            "address": issuer_address
        }
    }
    version = 1
    metadata = {'name': issuer_name, 'certNum': cert_num}
    pdf_utils.add_metadata(src_path, dest_path, version=version,
                           issuer=json.dumps(issuer), metadata=json.dumps(metadata),
                           chainpoint_proof='')


def calc_hash(file):
    with open(file, 'rb') as cert:
        hash_str = hashlib.sha256(cert.read()).hexdigest()
    return hash_str


def create_temporary_copy(path):
    temp_dir = tempfile.gettempdir()
    temp_path = os.path.join(temp_dir, 'temp_file_name')
    shutil.copy2(path, temp_path)
    return temp_path


def prepare_chainpoint_tree(hashes):
    cp = ChainPointV2()
    cp.add_leaf(hashes)
    cp.make_tree()
    return cp
